package application;

import debug.*;
import timer.*;
import link.*;
import network.*;
import reliability.*;

import java.io.*;
import java.util.*;

/**
  A class to act as an application to drive the sending end of a 
  bytestream conversation.  Matches up with a 
  SimpleBytestreamReceiver on the other end, and works through
  a ReliableSender instance to acheive communication.
  */

public class SimpleBytestreamSender extends TimerClient 
                                    implements Runnable
{
  private ReliableSender sender;
  private Address addr;
  private Address receiverAddr;
  private int numBytes;

  /**
    Creates a new sender client and starts the run thread for 
    it.  Clients should not be made to share sender instances,
    because it probably won't work.
    
    @param sendImplementation the ReliableSender instance
           (which needs to be connected to a network and have
           an address set) through which to transmit
	   
    @param address the address of this host.  Used to name
           the sending thread

    @param receiverAddress the address of the receiver, used
           to open the connection

    @param numBytes
           The number of bytes to send
	   */
  public SimpleBytestreamSender (ReliableSender sendImplementation, 
				 Address address, Address receiverAddress, 
				 int numBytes)
  {
    sender = sendImplementation;
    addr = address;
    receiverAddr = receiverAddress;

    this.numBytes = numBytes; 

    Thread sendThread = new Thread (this);
    Timer.global.RegisterThread (sendThread);
    sendThread.start ();
  }
  
  /*
    The sending thread.  Clients should not call this: the
    thread is started automatically by the constructor
    */
  public void run ()
  {
    Thread.currentThread ().setName ("Sender application at " + addr);
    synchronized (Timer.global)
      {
	sender.Open (receiverAddr);
	
	byte[] data = new byte[50];
	String s = "";
	int count = 0;
	int bytesSent = 0; 

	while (bytesSent < numBytes) {
	  while (s.length () < 50)
	    {
	      s = s + " " + count;
	      ++count;
	    }
	  
	  s.getBytes (0, 50, data, 0);
	  s = s.substring (50);
	  
	  sender.Send (data);
	  bytesSent += 50; 
	}

	Timer.global.UnregisterThread (Thread.currentThread ());
      }
  }
}

